/////////////////////////////////////////////////////////////////////////////
// Name:        prntbase.h
// Purpose:     Base classes for printing framework
// Author:      Julian Smart
// Modified by:
// Created:     01/02/97
// RCS-ID:      $Id: prntbase.h,v 1.11 1999/02/28 20:55:17 JS Exp $
// Copyright:   (c) Julian Smart and Markus Holzem
// Licence:   	wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_PRNTBASEH__
#define _WX_PRNTBASEH__

#ifdef __GNUG__
#pragma interface "prntbase.h"
#endif

#include "wx/defs.h"
#include "wx/event.h"
#include "wx/cmndata.h"
#include "wx/panel.h"
#include "wx/scrolwin.h"
#include "wx/dialog.h"
#include "wx/frame.h"

class WXDLLEXPORT wxDC;
class WXDLLEXPORT wxButton;
class WXDLLEXPORT wxChoice;
class WXDLLEXPORT wxPrintout;
class WXDLLEXPORT wxPrinterBase;
class WXDLLEXPORT wxPrintDialog;
class WXDLLEXPORT wxPrintPreviewBase;
class WXDLLEXPORT wxPreviewCanvas;
class WXDLLEXPORT wxPreviewControlBar;
class WXDLLEXPORT wxPreviewFrame;

/*
 * Represents the printer: manages printing a wxPrintout object
 */
 
class WXDLLEXPORT wxPrinterBase: public wxObject
{
  DECLARE_CLASS(wxPrinterBase)

public:
  wxPrinterBase(wxPrintData *data = (wxPrintData *) NULL);
  ~wxPrinterBase();

  virtual wxWindow *CreateAbortWindow(wxWindow *parent, wxPrintout *printout);
  virtual void ReportError(wxWindow *parent, wxPrintout *printout, char *message);
  inline wxPrintData& GetPrintData() const { return (wxPrintData&) m_printData; };
  inline bool GetAbort() const { return sm_abortIt; }

  ///////////////////////////////////////////////////////////////////////////
  // OVERRIDES

  virtual bool Setup(wxWindow *parent) = 0;
  virtual bool Print(wxWindow *parent, wxPrintout *printout, bool prompt = TRUE) = 0;
  virtual bool PrintDialog(wxWindow *parent) = 0;

protected:
  wxPrintData           m_printData;
  wxPrintout*           m_currentPrintout;
public:
  static wxWindow*      sm_abortWindow;
  static bool           sm_abortIt;

};

/*
 * wxPrintout
 * Represents an object via which a document may be printed.
 * The programmer derives from this, overrides (at least) OnPrintPage,
 * and passes it to a wxPrinter object for printing, or a wxPrintPreview
 * object for previewing.
 */
 
class WXDLLEXPORT wxPrintout: public wxObject
{
  DECLARE_ABSTRACT_CLASS(wxPrintout)

public:
  wxPrintout(const wxString& title = "Printout");
  ~wxPrintout();

  virtual bool OnBeginDocument(int startPage, int endPage);
  virtual void OnEndDocument();
  virtual void OnBeginPrinting();
  virtual void OnEndPrinting();

  // Guaranteed to be before any other functions are called
  inline virtual void OnPreparePrinting() { }

  virtual bool HasPage(int page);
  virtual bool OnPrintPage(int page) = 0;
  virtual void GetPageInfo(int *minPage, int *maxPage, int *pageFrom, int *pageTo);

  inline virtual wxString GetTitle() const { return m_printoutTitle; }

  inline wxDC *GetDC() const { return m_printoutDC; }
  inline void SetDC(wxDC *dc) { m_printoutDC = dc; }
  inline void SetPageSizePixels(int w, int  h) { m_pageWidthPixels = w; m_pageHeightPixels = h; }
  inline void GetPageSizePixels(int *w, int  *h) const { *w = m_pageWidthPixels; *h = m_pageHeightPixels; }
  inline void SetPageSizeMM(int w, int  h) { m_pageWidthMM = w; m_pageHeightMM = h; }
  inline void GetPageSizeMM(int *w, int  *h) const { *w = m_pageWidthMM; *h = m_pageHeightMM; }

  inline void SetPPIScreen(int x, int y) { m_PPIScreenX = x; m_PPIScreenY = y; }
  inline void GetPPIScreen(int *x, int *y) const { *x = m_PPIScreenX; *y = m_PPIScreenY; }
  inline void SetPPIPrinter(int x, int y) { m_PPIPrinterX = x; m_PPIPrinterY = y; }
  inline void GetPPIPrinter(int *x, int *y) const { *x = m_PPIPrinterX; *y = m_PPIPrinterY; }

  inline virtual bool IsPreview() const { return m_isPreview; }

  inline virtual void SetIsPreview(bool p) { m_isPreview = p; }

private:
   wxString         m_printoutTitle;
   wxDC*            m_printoutDC;

   int              m_pageWidthPixels;
   int              m_pageHeightPixels;

   int              m_pageWidthMM;
   int              m_pageHeightMM;

   int              m_PPIScreenX;
   int              m_PPIScreenY;
   int              m_PPIPrinterX;
   int              m_PPIPrinterY;

   bool             m_isPreview;
};

/*
 * wxPreviewCanvas
 * Canvas upon which a preview is drawn.
 */
 
class WXDLLEXPORT wxPreviewCanvas: public wxScrolledWindow
{
  DECLARE_CLASS(wxPreviewCanvas)

public:
  wxPreviewCanvas(wxPrintPreviewBase *preview, wxWindow *parent,
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "canvas");
  ~wxPreviewCanvas();

  void OnPaint(wxPaintEvent& event);

  // Responds to colour changes
  void OnSysColourChanged(wxSysColourChangedEvent& event);

private:
  wxPrintPreviewBase*       m_printPreview;

DECLARE_EVENT_TABLE()
};

/*
 * wxPreviewFrame
 * Default frame for showing preview.
 */

class WXDLLEXPORT wxPreviewFrame: public wxFrame
{
  DECLARE_CLASS(wxPreviewFrame)

public:
  wxPreviewFrame(wxPrintPreviewBase *preview, wxFrame *parent, const wxString& title = "Print Preview",
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = wxDEFAULT_FRAME_STYLE, const wxString& name = "frame");
  ~wxPreviewFrame();

  void OnCloseWindow(wxCloseEvent& event);
  virtual void Initialize();
  virtual void CreateCanvas();
  virtual void CreateControlBar();

protected:
  wxWindow*             m_previewCanvas;
  wxPreviewControlBar*  m_controlBar;
  wxPrintPreviewBase*   m_printPreview;

DECLARE_EVENT_TABLE()
};

/*
 * wxPreviewControlBar
 * A panel with buttons for controlling a print preview.
 * The programmer may wish to use other means for controlling
 * the print preview.
 */

#define wxPREVIEW_PRINT        1
#define wxPREVIEW_PREVIOUS     2
#define wxPREVIEW_NEXT         4
#define wxPREVIEW_ZOOM         8

#define wxPREVIEW_DEFAULT      wxPREVIEW_PREVIOUS|wxPREVIEW_NEXT|wxPREVIEW_ZOOM

// Ids for controls
#define wxID_PREVIEW_CLOSE      1
#define wxID_PREVIEW_NEXT       2
#define wxID_PREVIEW_PREVIOUS   3
#define wxID_PREVIEW_PRINT      4
#define wxID_PREVIEW_ZOOM       5

class WXDLLEXPORT wxPreviewControlBar: public wxPanel
{
  DECLARE_CLASS(wxPreviewControlBar)

public:
  wxPreviewControlBar(wxPrintPreviewBase *preview, long buttons,
    wxWindow *parent, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "panel");
  ~wxPreviewControlBar();

  virtual void CreateButtons();
  virtual void SetZoomControl(int zoom);
  virtual int GetZoomControl();
  inline virtual wxPrintPreviewBase *GetPrintPreview() const { return m_printPreview; }

  void OnPrint(wxCommandEvent& event);
  void OnWindowClose(wxCommandEvent& event);
  void OnNext(wxCommandEvent& event);
  void OnPrevious(wxCommandEvent& event);
  void OnZoom(wxCommandEvent& event);
  void OnPaint(wxPaintEvent& event);

protected:
  wxPrintPreviewBase*   m_printPreview;
  wxButton*             m_closeButton;
  wxButton*             m_nextPageButton;
  wxButton*             m_previousPageButton;
  wxButton*             m_printButton;
  wxChoice*             m_zoomControl;
  long                  m_buttonFlags;

DECLARE_EVENT_TABLE()
};

/*
 * wxPrintPreview
 * Programmer creates an object of this class to preview a wxPrintout.
 */
 
class WXDLLEXPORT wxPrintPreviewBase: public wxObject
{
  DECLARE_CLASS(wxPrintPreviewBase)

public:
  wxPrintPreviewBase(wxPrintout *printout, wxPrintout *printoutForPrinting = (wxPrintout *) NULL, wxPrintData *data = (wxPrintData *) NULL);
  ~wxPrintPreviewBase();

  virtual bool SetCurrentPage(int pageNum);
  inline int GetCurrentPage() const { return m_currentPage; };

  inline void SetPrintout(wxPrintout *printout) { m_previewPrintout = printout; };
  inline wxPrintout *GetPrintout() const { return m_previewPrintout; };
  inline wxPrintout *GetPrintoutForPrinting() const { return m_printPrintout; };

  inline void SetFrame(wxFrame *frame) { m_previewFrame = frame; };
  inline void SetCanvas(wxWindow *canvas) { m_previewCanvas = canvas; };

  inline virtual wxFrame *GetFrame() const { return m_previewFrame; }
  inline virtual wxWindow *GetCanvas() const { return m_previewCanvas; }

  // The preview canvas should call this from OnPaint
  virtual bool PaintPage(wxWindow *canvas, wxDC& dc);

  // This draws a blank page onto the preview canvas
  virtual bool DrawBlankPage(wxWindow *canvas, wxDC& dc);

  // This is called by wxPrintPreview to render a page into
  // a wxMemoryDC.
  virtual bool RenderPage(int pageNum);

  inline wxPrintData& GetPrintData() { return m_printData; }

  virtual void SetZoom(int percent);
  inline int GetZoom() const { return m_currentZoom; };

  inline int GetMaxPage() const { return m_maxPage; }
  inline int GetMinPage() const { return m_minPage; }

  inline bool Ok() const { return m_isOk; }
  inline void SetOk(bool ok) { m_isOk = ok; }

  ///////////////////////////////////////////////////////////////////////////
  // OVERRIDES

  // If we own a wxPrintout that can be used for printing, this
  // will invoke the actual printing procedure. Called
  // by the wxPreviewControlBar.
  virtual bool Print(bool interactive) = 0;

  // Calculate scaling that needs to be done to get roughly
  // the right scaling for the screen pretending to be
  // the currently selected printer.
  virtual void DetermineScaling() = 0;

protected:
  wxPrintData       m_printData;
  wxWindow*         m_previewCanvas;
  wxFrame*          m_previewFrame;
  wxBitmap*         m_previewBitmap;
  wxPrintout*       m_previewPrintout;
  wxPrintout*       m_printPrintout;
  int               m_currentPage;
  int               m_currentZoom;
  float             m_previewScale;
  int               m_topMargin;
  int               m_leftMargin;
  int               m_pageWidth;
  int               m_pageHeight;
  int               m_minPage;
  int               m_maxPage;
 protected:
  bool              m_isOk;
};

/*
 * Abort dialog
 */

class WXDLLEXPORT wxPrintAbortDialog: public wxDialog
{
public:
  wxPrintAbortDialog(wxWindow *parent,
    const wxString& title, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "dialog"):
   wxDialog(parent, -1, title, pos, size, style, name)
  {
  }

  void OnCancel(wxCommandEvent& event);

  DECLARE_EVENT_TABLE()
};

/*
 * Again, this only really needed for non-Windows platforms
 * or if you want to test the PostScript printing under Windows,
 * or if you're using the generic page setup dialog under e.g. Win16.
 */

class WXDLLEXPORT wxPrintPaperType: public wxObject
{
public:
    wxPrintPaperType(const char *name = (const char *) NULL, int wmm = 0, int hmm = 0, int wp = 0, int hp = 0);
    ~wxPrintPaperType();

public:
    int widthMM;
    int heightMM;
    int widthPixels;
    int heightPixels;
    char *pageName;

private:
    DECLARE_DYNAMIC_CLASS(wxPrintPaperType)
};

class WXDLLEXPORT wxPrintPaperDatabase: public wxList
{
public:
    wxPrintPaperDatabase();
    ~wxPrintPaperDatabase();

    void CreateDatabase();
    void ClearDatabase();

    void AddPaperType(const char *name, int wmm, int hmm, int wp, int hp);
    wxPrintPaperType *FindPaperType(const char *name);

private:
    DECLARE_DYNAMIC_CLASS(wxPrintPaperDatabase)
};

WXDLLEXPORT_DATA(extern wxPrintPaperDatabase*) wxThePrintPaperDatabase;


#endif
    // _WX_PRNTBASEH__
